/*

Print a priority queue in the format defined by the dot program.

The dot program is part of the graphics visualization tool set that
is available from ATT.  <http://www.research.att.com/sw/tools/graphviz/>

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "reporter.h"
#include "bheap.h"
#include "plotbheap.h"
#include "privatebheap.h"

#define MAX_LABEL_LENGTH 100

void plotBHeapHeader(PriorityQueue Q, char *title) {

  time_t t;			/* time of day */
  char *author;			/* author's name */
  char *timestring;		/* formatted time string */
  char plotLabel[MAX_LABEL_LENGTH+1]; /* label string */
  char *attributes = "size=\"10,7.5\";\ncenter=true;\nnode [shape=record];\n";

  author = getBHeapAuthor();

  t = time(NULL);
  timestring = ctime(&t);
  timestring[strlen(timestring)-1] = '\0';

  plotLabel[0] = '\0';
  if (title != NULL)
    strncat(plotLabel,title,MAX_LABEL_LENGTH-strlen(plotLabel));
  else
    strncat(plotLabel,"Binary Heap",MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel,"\\n",MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel,timestring,MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel," - ",MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel,author,MAX_LABEL_LENGTH-strlen(plotLabel));

  printf("/* binary heap */\n");
  printf("digraph %s {\n","G");
  printf("label=\"%s\";\n",plotLabel);
  printf("%s",attributes);
}

void plotBHeapFooter(PriorityQueue Q) {
  printf("} \n/* %i entries (capacity %i) */\n",Q->Size,Q->Capacity);
}

/*

Print a line that defines each node in the binary heap.

*/
void plotBHeapNodes(PriorityQueue Q,ElementPrintLabel P) {
  int i;
  ElementType e;

  for (i=1; i<Q->Size+1; i++) {
    e = Q->Element[i];
    printf("n%p[label = \"<f0>.| <f1> ",e);
    (*P)(e);
    printf("|<f2>.\" ];");
    printf("\t/* i: %i",i);
    printf(" */\n");
  }
}

/*

Print the lines that define links in the binary heap.  The node name is
the address of the object. 

*/
void plotBHeapEdges(PriorityQueue Q) {
  int i;

  if (Q == NULL || Q->Size == 0) return;
  for (i = 1; i <= Q->Size; i++) {
    if (i*2 <= Q->Size) {
      printf("\"n%p\":f0 -> \"n%p\":f1;\n", Q->Element[i], Q->Element[i*2] );
    }
    if (i*2+1 <= Q->Size) {
      printf("\"n%p\":f2 -> \"n%p\":f1;\n", Q->Element[i],Q->Element[i*2+1]);
    }
  }
}
